dnl Check if --with-expat[=PREFIX] is specified and
dnl Expat >= 1.95.0 is installed in the system.
dnl If yes, substitute EXPAT_CFLAGS, EXPAT_LIBS with regard to
dnl the specified PREFIX and set with_expat to PREFIX, or 'yes' if PREFIX
dnl has not been specified. Also HAVE_LIBEXPAT, HAVE_EXPAT_H are defined.
dnl If --with-expat has not been specified, set with_expat to 'no'.
dnl In addition, an Automake conditional EXPAT_INSTALLED is set accordingly.
dnl This is necessary to adapt a whole lot of packages that have expat
dnl bundled as a static library.
AC_DEFUN([AC_EXPAT], [ 
  AC_ARG_WITH([expat], 
              [  --with-expat=PREFIX     Use system Expat library],
              [expat_extra_dir=$with_expat], 
	            [expat_extra_dir=""])

  dnl Save context             
  AC_LANG_PUSH(C)
  expat_save_CFLAGS="$CFLAGS"
  expat_save_LIBS="$LIBS" 
  dnl Search for expat in one of the following dir 
  dirs="$expat_extra_dir /usr /usr/local none"
  expat_found=no
  for dir in $dirs ; do 
    if test "$expat_found" = "no" ; then 
      if test "$dir" = "none" ; then
        expat_test=
        expat_test_CFLAGS=
        expat_test_LIBS=
      else
        expat_test_base=$dir
        expat_test_CFLAGS=-I$dir/include
        expat_test_LIBS=-L$dir/lib
      fi 
	    AC_CHECK_LIB(expat, XML_ParserCreate,
		         [ EXPAT_LIBS="$expat_test_LIBS -lexpat"
		           expat_found=yes ],
		         [ expat_found=no ],
		         "$EXPAT_LIBS")
      AC_MSG_CHECKING([for -lexpat in $expat_test_LIBS])
      AC_MSG_RESULT([$expat_found])
      if test "$expat_found" = "yes"; then
	      CFLAGS="$CFLAGS $expat_test_CFLAGS"
	      AC_CHECK_HEADERS(expat.h, , expat_found=no)
        AC_MSG_CHECKING([for expat.h in $expat_test/include])
        AC_MSG_RESULT([$expat_found])
      fi	    
	  fi
  done
  dnl if $expat_found we've got it 
  if test "$expat_found" = "yes" ; then 
     HAVE_EXPAT="$expat_test"
     EXPAT_CFLAGS="$expat_test_CFLAGS"
     EXPAT_LIBS="$expat_test_LIBS -lexpat"
     AC_SUBST(HAVE_EXPAT)
     AC_SUBST(EXPAT_CFLAGS)
     AC_SUBST(EXPAT_LIBS)
     [$1]
  else 
     [$2]
	fi
  dnl restore context 
  AC_LANG_POP(C)
  CFLAGS="$expat_save_CFLAGS"
  LIBS="$expat_save_LIBS"
])
