dnl
dnl Define all macro related to XML parsing stuff 
dnl Most of this code is exacted from the CEGUI project 
dnl (http://cegui.org.uk) All credits are due to 
dnl Crazy Eddie (Aka Paul Turner)
dnl 

dnl ---------------------------------------------------------------------
dnl Parser detection 
dnl ---------------------------------------------------------------------

dnl ---------------------------------------------------------------------
dnl Check for Xerces-C availability 
dnl UTIL_CHECK_XERCES([prefix], [action-if-found], [action-if-not-found])
dnl ---------------------------------------------------------------------
AC_DEFUN([UTIL_CHECK_XERCES], [
    AC_ARG_WITH([xerces-incdir], AC_HELP_STRING([--with-xerces-incdir=DIR], [Optionally specifies location of the xerces-c++ includes]),
        	[util_xerces_incdir=$withval],[util_xerces_incdir=[.]])
    AC_ARG_WITH([xerces-libdir], AC_HELP_STRING([--with-xerces-libdir=DIR], [Optionally specifies location of the xerces-c++ library]),
        	[util_xerces_libdir=$withval],[util_xerces_libdir=[.]])
    util_lib_paths="$util_xerces_libdir "
    util_inc_paths="$util_xerces_incdir /usr/local/include/xerces /usr/include/xerces /usr/local/include /usr/include"

    $1_CFLAGS=
    $1_LIBS=

    util_saved_CFLAGS="$CPPFLAGS"
    util_saved_LIBS="$LIBS"

    for util_path in $util_inc_paths; do
        ifelse($util_path, [.], CPPFLAGS="$util_saved_CFLAGS", CPPFLAGS="-I$util_path $util_saved_CFLAGS")
        AC_PREPROC_IFELSE(
            [#include <xercesc/sax2/DefaultHandler.hpp>],
            [util_xerces_h_found=yes; util_xerces_flags="$util_path"; break],
            [util_xerces_h_found=no])
    done

    for util_path in $util_lib_paths; do
        ifelse($util_path, [.], LIBS="$util_saved_LIBS", LIBS="-L$util_path $util_saved_LIBS")
        AC_CHECK_LIB([xerces-c], [main],
            [util_xerces_l_found=yes; util_xerces_libs="$util_path"; break],
            [util_xerces_l_found=no;], -lpthread)
    done

    CPPFLAGS="$util_saved_CFLAGS"
    LIBS="$util_saved_LIBS"

    if test x$util_xerces_h_found = xyes && test x$util_xerces_l_found = xyes; then
        if test x$util_xerces_flags != x.; then
            $1_CFLAGS="-I$util_xerces_flags"
        fi
        if test x$util_xerces_libs = x.; then
            $1_LIBS="-lxerces-c"
        else
            $1_LIBS="-L$util_xerces_libs -lxerces-c"
        fi

        ifelse([$2], [], :, [$2])
    else
        ifelse([$3], [], :, [$3])
    fi
])
dnl ---------------------------------------------------------------------
dnl Check for Expat availability 
dnl UTIL_CHECK_EXPAT([prefix], [action-if-found], [action-if-not-found])
dnl ---------------------------------------------------------------------
AC_DEFUN([UTIL_CHECK_EXPAT], [
    AC_ARG_WITH([expat-incdir], AC_HELP_STRING([--with-expat-incdir=DIR], [Optionally specifies location of the expat-c++ includes]),
        [util_expat_incdir=$withval],[util_expat_incdir=[.]])
    AC_ARG_WITH([expat-libdir], AC_HELP_STRING([--with-expat-libdir=DIR], [Optionally specifies location of the expat-c++ library]),
        [util_expat_libdir=$withval],[util_expat_libdir=[.]])
    util_lib_paths="$util_expat_libdir"
    util_inc_paths="$util_expat_incdir /usr/local/include/expat /usr/include/expat /usr/local/include /usr/include"

    $1_CFLAGS=
    $1_LIBS=

    util_saved_CFLAGS="$CPPFLAGS"
    util_saved_LIBS="$LIBS"

    for util_path in $util_inc_paths; do
        ifelse($util_path, [.], CPPFLAGS="$util_saved_CFLAGS", CPPFLAGS="-I$util_path $util_saved_CFLAGS")
        AC_PREPROC_IFELSE(
            [#include <expat.h>],
            [util_expat_h_found=yes; util_expat_flags="$util_path"; break],
            [util_expat_h_found=no])
    done

    for util_path in $util_lib_paths; do
        ifelse($util_path, [.], LIBS="$util_saved_LIBS", LIBS="-L$util_path $util_saved_LIBS")
        AC_CHECK_LIB([expat], [XML_ParserCreate],
            [util_expat_l_found=yes; util_expat_libs="$util_path"; break],
            [util_expat_l_found=no; unset ac_cv_lib_expat-c_main])
    done

    CPPFLAGS="$util_saved_CFLAGS"
    LIBS="$util_saved_LIBS"

    if test x$util_expat_h_found = xyes && test x$util_expat_l_found = xyes; then
        if test x$util_expat_flags != x.; then
            $1_CFLAGS="-I$util_expat_flags"
        fi
        if test x$util_expat_libs = x.; then
            $1_LIBS="-lexpat"
        else
            $1_LIBS="-L$util_expat_libs -lexpat"
        fi

        ifelse([$2], [], :, [$2])
    else
        ifelse([$3], [], :, [$3])
    fi
])
dnl ---------------------------------------------------------------------
dnl Check for TinyXML availability (do nothing tiny is build in)
dnl UTIL_CHECK_TINY([prefix], [action-if-found], [action-if-not-found])
dnl ---------------------------------------------------------------------
AC_DEFUN([UTIL_CHECK_TINY], [
    $1_CFLAGS=
    $1_LIBS=
    [$2]
])
dnl ---------------------------------------------------------------------
dnl Global XML module macro 
dnl ---------------------------------------------------------------------

dnl ---------------------------------------------------------------------
dnl Configure XML supports 
dnl UTIL_XML([action-if-found],[action-if-not-found])
dnl ---------------------------------------------------------------------
AC_DEFUN([UTIL_XML], [
    util_have_xml=no
    util_have_xml_xerces=no
    util_have_xml_expat=no
    util_have_xml_tiny=no
   
dnl    UTIL_CHECK_XERCES(xerces, [util_found_xerces=yes], [util_found_xerces=no])
    UTIL_CHECK_EXPAT(expat, [util_found_expat=yes], [util_found_expat=no])
    UTIL_CHECK_TINY(tiny, [util_found_tiny=yes], [util_found_tiny=no])	


    # Select active parsers 
    AC_ARG_ENABLE([xml], 
	    AC_HELP_STRING([--disable-xml], [Disable XML support and all class using it (default: enable)]), 
	    [util_have_xml=$enableval], [util_have_xml=yes])
dnl    AC_ARG_ENABLE([xerces],
dnl	    AC_HELP_STRING([--disable-xerces], [Do not build Xerces-C++ parser (default: auto)]), 
dnl	    [util_have_xerces=$enableval], [util_have_xerces=yes])
    AC_ARG_ENABLE([expat],
	    AC_HELP_STRING([--disable-expat], [Do not build Expat parser (default: auto)]), 
	    [util_have_expat=$enableval], [util_have_expat=yes])		
    AC_ARG_ENABLE([tinyxml], 
	    AC_HELP_STRING([--disable-tinyxml], [Do not build TinyXML built-in parser (default: enable)]), 
	    [util_have_tiny=$enableval], [util_have_tiny=yes])    
    # Set Default parser
    AC_ARG_WITH([xml-default-parser], 
	    AC_HELP_STRING([--with-default-xml-parser], [Set the default XML Parser]), 
	    [util_default_xml_parser=$withval], [util_default_xml_parser=ExpatXMLParser])
			
    # No XML Support in the utility library 
    if test "$util_have_xml" = "yes" ; then
	    # Enable the various parser 
	    util_xml_parser_list=""
    	# Check for Xerces 
	    if test "$util_have_xerces" = "yes" -a "$util_found_xerces" = "yes" ; then
            util_xml_parser_list="XercesXMLParser $util_xml_parser_list"
	        XERCES_CFLAGS="$xerces_CFLAGS"
    	    XERCES_LIBS="$xerces_LIBS"
 	        util_have_xml_xerces=yes
            AC_SUBST(XERCES_CFLAGS)
	        AC_SUBST(XERCES_LIBS)
            AM_CONDITIONAL(UTIL_OPT_XML_XERCES, true)
            AC_DEFINE_UNQUOTED(UTIL_OPT_HAVE_XML_XERCES, [1], [Xerces parser support is enabled])
	    else
	        AM_CONDITIONAL(UTIL_OPT_XML_XERCES, false)
	    fi 
        
    	# Check for Expat 
	    if test "$util_have_expat" = "yes" -a "$util_found_expat" = "yes" ; then 
	        util_xml_parser_list="ExpatXMLParser $util_xml_parser_list"
    	    EXPAT_CFLAGS="$expat_CFLAGS"
	        EXPAT_LIBS="$expat_LIBS"
	        util_have_xml_expat=yes
            AC_SUBST(EXPAT_CFLAGS)
            AC_SUBST(EXPAT_LIBS)
	        AM_CONDITIONAL(UTIL_OPT_XML_EXPAT, true)
	        AC_DEFINE_UNQUOTED(UTIL_OPT_HAVE_XML_EXPAT, [1], [Expat parser support is enabled])
	    else
	        AM_CONDITIONAL(UTIL_OPT_XML_EXPAT, false)
	    fi 
        
	    # Check for TinyXML 
    	if test "$util_have_tiny" = "yes" -a "$util_found_tiny" = "yes" ; then 
	        util_xml_parser_list="TinyXMLParser $util_xml_parser_list"
	        TINY_CFLAGS="$tiny_CFLAGS"
    	    TINY_LIBS="$tiny_LIBS"
	        util_have_xml_tiny=yes
            AC_SUBST(TINY_CFLAGS)
            AC_SUBST(TINY_LIBS)
            AM_CONDITIONAL(UTIL_OPT_XML_TINY, true)
    	    AC_DEFINE_UNQUOTED(UTIL_OPT_HAVE_XML_TINY, [1], [TinyXML parser support is enabled])
	    else 
	        AM_CONDITIONAL(UTIL_OPT_XML_TINY, false)
	    fi 
    	# If at least one parser is available the activate XML subsystem 
	    if test "x$util_xml_parser_list" != "x" ; then
    	    AC_DEFINE_UNQUOTED(UTIL_OPT_HAVE_XML, [1], [XML module is enabled])
	        AM_CONDITIONAL(UTIL_OPT_HAVE_XML, true)
	        # Detect default parser 
	        util_default_xml_parser=`echo $util_default_xml_parser | tr "[[:upper:]]" "[[:lower:]]"`
    	    case "$util_default_xml_parser" in 
    	    	(xerces-c|xerces|xercesxmlparser)
	    		    util_default_xml_parser="XercesXMLParser"
			        ;;
    	    	(expat|expatxmlparser)
	    		    util_default_xml_parser="ExpatXMLParser"
		    	    ;;
    		    (tiny|tinyxml|tinyxmlparser)
	    		    util_default_xml_parser="TinyXMLParser"
		    	    ;;
    	    esac
	        util_xml_default_parser_found=no
 	        for name in $util_xml_parser_list ; do 
		        if test "$name"	= "$util_default_xml_parser" ; then
		            util_xml_default_parser_found=yes
		        fi
    	    done 
	        if test $util_xml_default_parser_found = "no" ; then 
		        util_default_xml_parser=`echo $util_xml_parser_list | cut '-d ' -f1`
    	    fi
            case $util_default_xml_parser in
                (XercesXMLParser)
                    AC_DEFINE(UTIL_OPT_XML_DEFAULT_PARSER, "XercesXMLParser", [Default parser name])
                    ;;
                (ExpatXMLParser)
                    AC_DEFINE(UTIL_OPT_XML_DEFAULT_PARSER, "ExpatXMLParser", [Default parser name])
                    ;;
                (*)
                    AC_DEFINE(UTIL_OPT_XML_DEFAULT_PARSER, "TinyXMLParser", [Default parser name])
                    ;;
            esac 
	        util_have_xml=yes
            [$1]
    	else 
	        AM_CONDITIONAL(UTIL_OPT_HAVE_XML, false)
	        [$2]
    	fi 
    else 
	    AM_CONDITIONAL(UTIL_OPT_XML_XERCES, false)
    	AM_CONDITIONAL(UTIL_OPT_XML_EXPAT, false)
	    AM_CONDITIONAL(UTIL_OPT_XML_TINY, false)
	    AM_CONDITIONAL(UTIL_OPT_HAVE_XML, false)
        [$2]
    fi
])
dnl ---------------------------------------------------------------------
dnl Summarize the configuration of the XML module 
dnl UTIL_XML_DISPLAY_CONIFG
dnl ---------------------------------------------------------------------
AC_DEFUN([UTIL_XML_DISPLAY_CONFIG], 
[
echo         "  XML Module ......................: $util_have_xml"
if test "$util_have_xml" = "yes" ; then
#	echo "    Xerces-C Parser ...............: $util_have_xml_xerces"
	echo "    Expat Parser ..................: $util_have_xml_expat"
	echo "    TinyXML Parser ................: $util_have_xml_tiny"
	echo "    Default Parser ................: $util_default_xml_parser"
fi
])
dnl ---------------------------------------------------------------------	
