/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/**
*******************************************************************************
* Utility Library 
*******************************************************************************
* @ingroup util_application
* @file
* @brief Abstract Class ApplicationAdaptor declaration
*  
* - Supports: All
* - Created: 2005-08-06
* - Creator: Olivier Delannoy
* - Changelog: 
*    * 2005-08-06: Moved to Util  
*    * 2005-07-08: Initial version  
*******************************************************************************
Copyright Olivier Delannoy 2005

Olivier Delannoy <Olivier.Delannoy@gmail.com>

This software is a computer program whose purpose is to [describe
functionalities and technical features of your software].

This software is governed by the CeCILL  license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

*******************************************************************************/
#ifndef UTIL_APPLICATIONADAPTOR_HH
#define UTIL_APPLICATIONADAPTOR_HH 1
#include "util_namespace.hh"
#include <iostream>
namespace Util {
    /**
     * @ingroup util_application
     * @brief This class hides the differencies between all kind
     * of applications. 
     * 
     * Using class derived from this one there's now some easy
     * ways to support transparenlty graphical or text based
     * application. The application adaptor deals with all
     * specific behaviour such as handling standard stream,
     * displaying error message, etc. For example a Gui
     * Application will start a console window on unix dedicated to the
     * display of all messages and will provide an input line as
     * stdin. While on linux we will use the default features. 
     */
    class UTIL_EXPORT ApplicationAdaptor
    {
    public:
        virtual ~ApplicationAdaptor() {}
        /**
         * Provides access to the standard input stream 
         */
        virtual std::istream& in(void)=0;
        /**
         * Provides access to the standard output stream 
         */
        virtual std::ostream& out(void)=0;
        /**
         * Provides access to the standard output stream 
         */
        virtual std::ostream& err(void)=0;
        /**
         * Display a notification message 
         * @param message the text to display 
         * @param type the type of message 
         */
        virtual void displayNotification(const String& message, 
                                         const NotificationType type = UTIL_NOTIFICATION_DEFAULT)=0;
            
    protected:

    private:
            
    };
}
#endif
/*******************************************************************************/      



