/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/**
 *******************************************************************************
 * Utility Library 
 *******************************************************************************
 * @file
 * @ingroup util_base
 * @brief Class ByteArray declaration
 *  
 * 
 * - Supports: All  
 * - Created: 2006-03-24
 * - Creator: Olivier Delannoy
 * - Last change in revision : $Revision $
 * - Changelog: 
 *    
 *    * 2006-03-24 : Initial version  
 ******************************************************************************
Copyright Olivier Delannoy 2005

Olivier Delannoy <Olivier.Delannoy@gmail.com>

This software is a computer program whose purpose is to [describe
functionalities and technical features of your software].

This software is governed by the CeCILL  license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

 *******************************************************************************/
#ifndef UTIL_BYTEARRAY_HH
#define UTIL_BYTEARRAY_HH 1
#include "util_namespace.hh"
#include <Exception.hh>
namespace Util 
{
    /**
     * @ingroup util_base
     * @brief raw bytes container. 
     *
     * This class is used to store raw data. 
     * 
     * - Supports: All
     * - Author: Olivier Delannoy
     * - Creator: 2006-03-24
     * - $Revision $
     */
    class UTIL_EXPORT ByteArray
    {
    public:
        /**
         * Constructor 
         * Create a byte array of @em size byte;
         */
        explicit ByteArray(uint32 size) throw(std::bad_alloc);
        /**
         * Constructor 
         * Create a byte array filled with the content of a file 
         */
        explicit ByteArray(const String& filename) throw(std::bad_alloc,IOException);
        
        /**
         * Destructor 
         */
        ~ByteArray(void);
        
        uint32 size(void) const;
        const byte* const data(void) const;
        byte* data(void);
        void resize(uint32 newSize = 0);
        
    protected:
    private:
        /** Store the size of the container */
        uint32 mSize;
        /** pointer to the byte array */ 
        byte* mData;
        /** 
         * Copy constructor 
         * @param src a reference to the object to copy 
         */
        ByteArray(ByteArray& src);        
        /**
         * Assignment operator 
         *
         * @param src a reference to the object to assign from 
         * 
         * @return a reference to the current object 
         */
        ByteArray& operator=(ByteArray& src);
    };
}
#ifdef UTIL_OPT_INLINE
#include "ByteArray.icc"
#endif
#endif // UTIL_BYTEARRAY_HH
/*******************************************************************************/



