/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/**
 *******************************************************************************
 * Utility Library 
 *******************************************************************************
 * @file
 * @brief Class CSVHandler declaration
 *  
 * 
 * - Supports: All  
 * - Created: 2006-03-03
 * - Creator: Olivier Delannoy
 * - Last change in revision : $Revision $
 * - Changelog: 
 *    
 *    * 2006-03-03 : Initial version  
 ******************************************************************************
Copyright Olivier Delannoy 2005

Olivier Delannoy <Olivier.Delannoy@gmail.com>

This software is a computer program whose purpose is to [describe
functionalities and technical features of your software].

This software is governed by the CeCILL  license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

 *******************************************************************************/
#ifndef UTIL_CSVHANDLER_HH
#define UTIL_CSVHANDLER_HH 1
#include "util_namespace.hh"
#include <iostream>
namespace Util 
{
    /**
     * @brief The CSV Handler is used during the parsing of csv files 
     *
     * This class is used with the CVSParser during the parsing of CVS
     * file.  The user must define the event he needs to create the
     * representation he needs.
     *
     *
     * - Supports: All
     * - Author: Olivier Delannoy
     * - Creator: 2006-03-03
     * - $Revision $
     */
    class UTIL_EXPORT CSVHandler
    {
    public:
        /**
         * Constructor 
         */
        CSVHandler(void);
        /**
         * Destructor 
         */
        virtual ~CSVHandler(void);
        /**
         * This event is triggered each time a new row begins 
         */
        virtual void rowStart();
        /** 
         * This event is triggered each time a row terminantes 
         */
        virtual void rowEnd();
        /**
         * This event is triggered each time a cell in a row is parsed 
         * @param text the content of the cell 
         */
        virtual void cell(const String& text);
    protected:
    
    private:
        /* Disabled operations */ 
        CSVHandler(CSVHandler& src);
        /* Disabled operations */ 
        CSVHandler& operator=(CSVHandler& src);        
    };
}
#endif // UTIL_CSVHANDLER_HH
/*******************************************************************************/



