/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/**
 *******************************************************************************
 * Utility Library 
 *******************************************************************************
 * @file
 * @brief Class CSVSerializer declaration
 *  
 * 
 * - Supports: All  
 * - Created: 2006-11-06
 * - Creator: Olivier Frebourg
 * - Last change in revision : $Revision $
 * - Changelog: 
 *    
 *    * 2006-11-06 : Initial version  
 ******************************************************************************
Copyright Olivier Delannoy 2005

Olivier Delannoy <Olivier.Delannoy@gmail.com>

This software is a computer program whose purpose is to [describe
functionalities and technical features of your software].

This software is governed by the CeCILL  license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

 *******************************************************************************/
#ifndef UTIL_CSVSERIALIZER_HH
#define UTIL_CSVSERIALIZER_HH 1
#include "util_namespace.hh"
#include <iostream>
namespace Util 
{
    /**
     * @brief Column Separated Value document creation
     * 
     * This class is used to create valid CSV document. 
     * 
     * - Supports: All
     * - Author: Olivier Delannoy
     * - Creator: 2006-11-06
     * - $Revision $
     */
    class UTIL_EXPORT CSVSerializer
    {
    public:
        /**
         * Constructor 
         */
        CSVSerializer(std::ostream& out, char separator = ',');
        /**
         * Destructor 
         */
        ~CSVSerializer(void);
        /**
         * Start a new row 
         * 
         * @return a reference to the current serialization object 
         */
        CSVSerializer& newRow();
        /**
         * Add a cell of text 
         * 
         * @param text the value of the cell 
         * 
         * @return a reference to the current serialization object 
         */
        CSVSerializer& cell(const String& text);
    protected:
    private:
        std::ostream& mStream;  //<! Store a reference to the stream 
        const char mSeparator;  //<! Store the cell separator 
        bool mStartOfRow;        //<! Store wether we are at the start or at the end of a row 
        
        // Disabled operation 
        CSVSerializer(CSVSerializer& src);
        CSVSerializer& operator=(CSVSerializer& src);
    };
}
#endif // UTIL_CSVSERIALIZER_HH
/*******************************************************************************/



