/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/**
 *******************************************************************************
 * Utility Library 
 *******************************************************************************
 * @file
 * @brief Class ConfigXCF declaration
 *  
 * 
 * - Supports: All  
 * - Created: 2005-08-10
 * - Creator: Olivier Delannoy
 * - Last change in revision : $Revision $
 * - Changelog: 
 *    
 *    * 2005-08-10 : Initial version  
 ******************************************************************************
Copyright Olivier Delannoy 2005

Olivier Delannoy <Olivier.Delannoy@gmail.com>

This software is a computer program whose purpose is to [describe
functionalities and technical features of your software].

This software is governed by the CeCILL  license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

 *******************************************************************************/
#ifndef UTIL_CONFIGXCF_HH
#define UTIL_CONFIGXCF_HH 1
#include "util_namespace.hh"
#include "Config.hh"
#include <iostream>

namespace Util 
{
    /**
     * @brief XML Based configuration file 
     *
     * The XCF file format was defined in this project. It provides
     * the same features than an INI based file format. Entries are
     * organized in groups each group have a name. An empty file is
     * shown bellow: 
     * @code 
     * <?xml version="1.0" ?> 
     * <config>
     *  <entry name="name" value="value" />
     *  <group name="name of group">
     *    <entry name="name" value="value" />
     *  </group>
     * </config> 
     * @endcode 
     *
     * This class implements both reading and writing of configuration
     * file in XCF format.
     * 
     * - Supports: All
     * - Author: Olivier Delannoy
     * - Creator: 2005-08-10
     * - $Revision $
     */
    class UTIL_EXPORT ConfigXCF : public Config
    {
    public:
        /**
         * Constructor 
         */
        ConfigXCF(void);
        /**
         * Destructor 
         */
        virtual ~ConfigXCF(void);
        /**
         * @note current implementation does not copy internal states... 
         */
        virtual Config* clone(void) const;
        
        /** Store the file extension associated to this configuration importer and exporter */
        static const String FILE_EXTENSION;
        /** Store the name of the schema file */
        static const String XSD_NAME;
    protected:
        /* Inherited doc */
        virtual void loadImpl(const String& fileName);
        /* Inherited doc */
        virtual void saveImpl(const String& fileName, bool overwrite); 
    private:
    };
}
#endif // UTIL_CONFIGXCF_HH
/*******************************************************************************/



