/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/**
*******************************************************************************
* Utility Library 
*******************************************************************************
* @ingroup util_application
* @file
* @brief Abstract Class ConsoleApplicationAdaptor declaration
*  
* - Supports: 
*   - system
*     - UNIX (Linux) 
*
* - Created: 2005-08-06
* - Creator: Olivier Delannoy
* - Changelog: 
*    * 2005-08-06: Moved to Util  
*    * 2005-07-08: Initial version  
*******************************************************************************
Copyright Olivier Delannoy 2005

Olivier Delannoy <Olivier.Delannoy@gmail.com>

This software is a computer program whose purpose is to [describe
functionalities and technical features of your software].

This software is governed by the CeCILL  license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

*******************************************************************************/
#ifndef UTIL_CONSOLEAPPLICATIONADAPTOR_HH
#define UTIL_CONSOLEAPPLICATIONADAPTOR_HH 1
#include "ApplicationAdaptor.hh"
namespace Util {
    
    /**
     * @ingroup util_application
     * @brief Provide a text based  facilities for applications
     *
     * This class implements all text based facilities for
     * applications. It reads its input on the command line, write
     * its output to the defaults streams.
     */
    class UTIL_EXPORT ConsoleApplicationAdaptor : public ApplicationAdaptor
    {
    public:
        /** 
         * Default constructor
         */
        ConsoleApplicationAdaptor(bool MergeOutErr = false);
        /**
         * Destructor
         */
        virtual ~ConsoleApplicationAdaptor(void);
        /**
         * Provides access to the input stream 
         */
        virtual std::istream& in(void);
        /**
         * Provides access to the output stream 
         */
        virtual std::ostream& out(void);
        /**
         * Provides access to the error stream 
         */
        virtual std::ostream& err(void);
        /** 
         * Notification message information 
         */
        virtual void displayNotification(const String& message, const NotificationType type);
            
    protected:

    private:
        /**
         * Input stream pointer 
         */
        std::istream *mIn;
        /**
         * Output stream pointer
         */ 
        std::ostream *mOut;
        /**
         * Error stream pointer 
         */
        std::ostream *mErr;      
    };
}
# ifdef UTIL_OPT_INLINE
#  include "ConsoleApplicationAdaptor.icc"
# endif
#endif
/******************************************************************************/



