/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/**
 *******************************************************************************
 * Utility Library 
 *******************************************************************************
 * @file
 * @brief Class DirHandler declaration
 *  
 * 
 * - Supports: All  
 * - Created: 2006-10-29
 * - Creator: Olivier Delannoy
 * - Last change in revision : $Revision $
 * - Changelog: 
 *    
 *    * 2006-10-29 : Initial version  
 ******************************************************************************
Copyright Olivier Delannoy 2005

Olivier Delannoy <Olivier.Delannoy@gmail.com>

This software is a computer program whose purpose is to [describe
functionalities and technical features of your software].

This software is governed by the CeCILL  license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

 *******************************************************************************/
#ifndef UTIL_DIRHANDLER_HH
#define UTIL_DIRHANDLER_HH 1
#include "util_namespace.hh"
#include <iostream>
namespace Util 
{
    /**
     * @brief 
     *
     * 
     * - Supports: All
     * - Author: Olivier Delannoy
     * - Creator: 2006-10-29
     * - $Revision $
     */
    class UTIL_EXPORT DirHandler
    {
    public:
        /**
         * Constructor 
         */
        DirHandler(void);
        /**
         * Destructor 
         */
        virtual ~DirHandler(void);
        /** 
         * @brief 
         * Define this method if you want to deal with files available in a directory tree 
         * 
         * This method is called each time a file is encountered in a
         * directory traversal. You have to define your version of
         * this handler in order to be able to do something with this
         * file name.
         * 
         * @param parentPath the name of the directory containing the
         * file with a leading path separator
         *
         * @param filename the name of the file itself 
         *
         */
        virtual void onFile(const String& parentPath, const String& filename);
        /** 
         * @brief 
         * Define this method if you want to apply a special operation to a directory 
         * 
         * This method is called each time a directory is entered 
         * in a directory trasversal. You have to define your version of
         * this handler in order to be able to do something with this
         * file. 
         *
         * @param parentPath the name of the directory containing the
         * current directory with a leading path separator.
         *
         * @param dirname the name of the current directory. 
         */
        virtual void onEnterDir(const String& parentPath, const String& dirame); 
        /**
         * @brief 
         * Define this method if you want to apply a special operation to a directory 
         * 
         * This method is called each time a directory is leaved
         * in a directory trasversal. You have to define your version of
         * this handler in order to be able to do something with this
         * file. 
         *
         * @param parentPath the name of the directory containing the
         * current directory with a leading path separator.
         *
         * @param dirname the name of the current directory. 
         */
        virtual void onExitDir(const String& parentPath, const String& dirame);
    protected:
    
    private:
        DirHandler(DirHandler& src);
        DirHandler& operator=(DirHandler& src);
        
    };
}
#endif // UTIL_DIRHANDLER_HH
/*******************************************************************************/



