/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/**
*******************************************************************************
* Utility Library 
*******************************************************************************
* @ingroup util_template
* @file
* @brief Template Factory definition 
*  
* - Supports: All
* - Created: 2005-08-06
* - Creator: Olivier Delannoy
* - $Revision $
* - Changelog: 
*    * 2005-08-07 : Integration into libutil 
*        - Moved from Util::TL to Util namespace
*    * 2005-05-31 : Initial version  
*******************************************************************************
Copyright Olivier Delannoy 2005

Olivier Delannoy <Olivier.Delannoy@gmail.com>

This software is a computer program whose purpose is to [describe
functionalities and technical features of your software].

This software is governed by the CeCILL  license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

*******************************************************************************/
#ifndef UTIL_FACTORY_HH
#define UTIL_FACTORY_HH 1
# include "Singleton.hh"
# include <map>
namespace Util {
    /**
     * @ingroup util_template
     * @brief A factory is an object creator. 
     * 
     * A factory is an object creator. It's goal consists in providing
     * objects on demands. This template define a special kind of
     * factory. The object created by those factory must be shared and
     * be accesible throught a unique key. The Factory is also in
     * charge of managing object created. Each object can be accessed
     * throught it's pointer or a reference. 
     *
     */
    template <class T>
    class Factory : public Singleton<Factory<T> >
    {
    public:
        typedef T element_type;
        typedef String key_type;
            
        /**
         * Default Constructor
         */
        Factory(void)
        {
                
        }
            
        /**
         * Destructor 
         */
        virtual ~Factory(void) 
        {
            //Destroy all manage objects 
            iterator_type iter;
                
            for(iter = mObjects.begin(); iter != mObjects.end() ; ++iter)
                delete iter->second;
        }
            
        /**
         * Create the shared object on the first request or return
         * a pointer to the existing object. The factory is in
         * charge of managing the memory of the created object.
         * @param name the name of the object to create 
         * @return A pointer to the shared object 
         */
        T* create(const key_type& name) 
        {
            iterator_type iter = mObjects.find(name);
            if (iter != mObjects.end())
            {
                return iter->second;
            }
            else 
            {
                //Create and initialise objects 
                element_type* tmp = createObject(name);
                if (tmp)
                {
                    mObjects[name] = tmp;
                    return tmp;    
                }
            }
            return 0;
        }    
        using Singleton<Factory<element_type> >::getSingleton;
        using Singleton<Factory<element_type> >::getSingletonPtr;  
    protected:
        virtual element_type* createObject(const key_type& name) = 0;
        typedef typename std::map<key_type, element_type*>::iterator iterator_type;    
        /** Store ojects */
        std::map<key_type, element_type*> mObjects;                

    private:
    };
}
#endif
/*******************************************************************************/      



