/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/**
 *******************************************************************************
 * Utility Library 
 *******************************************************************************
 * @file
 * @ingroup util_system
 * @brief Class File declaration
 *  
 * 
 * - Supports: All  
 * - Created: 2005-08-24
 * - Creator: Olivier DELANNOY
 * - Last change in revision : $Revision $
 * - Changelog: 
 *    
 *    * 2005-08-24 : Initial version  
 ******************************************************************************
Copyright Olivier Delannoy 2005

Olivier Delannoy <Olivier.Delannoy@gmail.com>

This software is a computer program whose purpose is to [describe
functionalities and technical features of your software].

This software is governed by the CeCILL  license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

 *******************************************************************************/
#ifndef UTIL_FILE_HH
#define UTIL_FILE_HH 1
#include "util_namespace.hh"
#include <iostream>
namespace Util 
{
    /**
		 * @ingroup util_system
     * @brief File manipulation class 
     * 
     * This class provide some features concerning file operation such as 
     * - moving 
     * - removing 
     * 
     * - Supports:
     *   - system 
     *     - UNIX: (linux)
     * - Author: Olivier DELANNOY
     * - Creator: 2005-08-24
     * - $Revision $
     */
    class UTIL_EXPORT File
    {
    public:
        /** 
         * Delete a file from the filesystem 
         * @param fileName the path of the file to remove
         * @return true on success false otherwise 
         */
        static bool remove(const String& fileName);
        /**
         * Rename or move a file
         * @param dstName the new path of the file 
         * @param srcName the old path of the file 
         * @return true on success false otherwise 
         */
        static bool rename(const String& dstName, 
                           const String& srcName);
        /** 
         * Create a copy of an existing file 
         * @param dstName the target path 
         * @param srcName the name of the file to be copied 
         * @return true on success false otherwise
         *
         * @note if an error occured during the process the target 
         * path is removed.
         */
        static bool copy(const String& dstName, const String& srcName);
        
        /**
         * Create a temporary path name valid to create temporary files
         * 
         * @return the absolute path to the temporary file 
         *
         * @note this function does not open a file.
         */
        static String tempName();
        
    protected:
    private:
    };
}
#endif // UTIL_FILE_HH
/*******************************************************************************/



