/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/**
 *******************************************************************************
 * Utility Library 
 *******************************************************************************
 * @file
 * @brief Class Index declaration
 *  
 * 
 * - Supports: All  
 * - Created: 2006-03-17
 * - Creator: Olivier Delannoy
 * - Last change in revision : $Revision $
 * - Changelog: 
 *    
 *    * 2006-03-17 : Initial version  
 ******************************************************************************
Copyright Olivier Delannoy 2005

Olivier Delannoy <Olivier.Delannoy@gmail.com>

This software is a computer program whose purpose is to [describe
functionalities and technical features of your software].

This software is governed by the CeCILL  license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

 *******************************************************************************/
#ifndef UTIL_INDEX_HH
#define UTIL_INDEX_HH 1
#include "util_namespace.hh"
#include <iostream>
namespace Util 
{
    /**
     * @brief An Index is a class used to create quick access to a data collection.
     *
     * This class is used to index a collection of data. Several index
     * can manage the same collection but are not linked altogether.
     * The removal of a value of the collection must be transmitted to all index. 
     * - Supports: All 
     * - Author: Olivier Delannoy 
     * - Creator: 2006-03-17 
     * - $Revision $
     */
    template <typename TKey, typename TValue>
    class UTIL_EXPORT Index
    {
    public:
        /**
         * Typename for the key 
         */ 
        typedef TKey key_type;
        /**
         * Typename for the value which is an entry in the collection (ex: integer) 
         */
        typedef TValue value_type;
        /**
         * Constructor
         */
        Index(void) {}
        /**
         * Destructor
         */
        virtual ~Index(void) {}
        /**
         * Insert a new value in the index 
         * @param key the key to be inserted 
         * @param value the value associated to key 
         */
        virtual void insert(const key_type& key, const value_type& value) = 0;
        /** 
         * Remove a value from the index 
         * @param key The key to be removed from the index. 
         *
         * @note some implementation cannot provide the operation
         * which does nothing in this case.
         */
        virtual void remove(const key_type& key) 
        {
        }
        /**
         * Find the value associated to a key 
         * @param key the key to be search for 
         * 
         * @return the value if found else Value::NULL.
         */
        virtual const value_type& find(const key_type& key) = 0;

        /** 
         * Retrieve the number of element references in the index 
         * @return the number of element available 
         */
         virtual uint32 size(void) const = 0;
    protected:
        
    private:
        Index& operator=(Index& obj);
        Index(Index& obj);
    };
}
#endif // UTIL_INDEX_HH
/*******************************************************************************/



