/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/**
*******************************************************************************
* Utility Library 
*******************************************************************************
* @ingroup util_template
* @file
* @brief Template NullStreambuf definition 
*  
* - Supports: All
* - Created: 2005-08-06
* - Creator: Olivier Delannoy
* - $Revision $
* - Changelog: 
*    * 2005-08-07 : Integration into libutil 
*        - Moved from Util::TL to Util namespace
*    * 2005-05-31 : Initial version  
*******************************************************************************
Copyright Olivier Delannoy 2005

Olivier Delannoy <Olivier.Delannoy@gmail.com>

This software is a computer program whose purpose is to [describe
functionalities and technical features of your software].

This software is governed by the CeCILL  license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

*******************************************************************************/
#ifndef UTIL_NULLSTREAMBUF_HH
#define UTIL_NULLSTREAMBUF_HH 1
#include <streambuf>
namespace Util {
    /**
     * @ingroup util_template
     * @brief Provide a streambuffer that eat any character 
     * 
     * This streambuf ignore all writes to it's internal buffer and
     * discard any receive information. It's a do nothing streambuf to 
     * plug to an ostream. 
     * 
     * @code
     * #include "NullStreambuf.hh"
     * typedef Util::NullStreambuf<char> eater;
     * int main(void) 
     * {
     *     eater buffer;
     *     std::ostream out(&buffer);
     *     out << "toto" << std::endl; // Nothing append 
     *     return 0;
     * }
     * @endcode
     */
    template <typename _CharT, typename _Traits = std::char_traits<_CharT> > 
    class NullStreambuf : public std::basic_streambuf<_CharT, _Traits>
    {
    public:
        typedef _CharT char_type;
        typedef _Traits traits_type;
        typedef typename traits_type::int_type int_type;
        typedef typename traits_type::pos_type pos_type;
        typedef typename traits_type::off_type off_type;
        typedef std::size_t size_t;
    

        /**
         * Default Constructor
         */
        NullStreambuf(void)
        {
            this->setbuf(0,0);
            //_M_mode = std::ios_base::out;
        }
            
        /**
         * Destructor 
         */
        virtual ~NullStreambuf(void) 
        {
        }
            
    protected:
    private:
        virtual int_type overflow(int_type  __c = traits_type::eof()) 
        {
            return traits_type::not_eof(__c);   
        }
    };
}
#endif
/*******************************************************************************/      




