/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/**
*******************************************************************************
* Utility Library 
*******************************************************************************
* @ingroup util_tools 
* @file
* @brief Class Pack declaration
*  
* 
* - Supports: 
*   - UNIX: (POSIX) 
* - Created: 2005-08-25
* - Creator: Olivier DELANNOY
* - Last change in revision : $Revision $
* - Changelog: 
*    
*    * 2005-08-25 : Initial version  
******************************************************************************
Copyright Olivier Delannoy 2005

Olivier Delannoy <Olivier.Delannoy@gmail.com>

This software is a computer program whose purpose is to [describe
functionalities and technical features of your software].

This software is governed by the CeCILL  license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

*******************************************************************************/
#ifndef UTIL_PACK_HH
#define UTIL_PACK_HH 1
#include "util_namespace.hh"
#include <vector>
#include "ByteArray.hh" 
namespace Util {
    /**
     * @brief Group or Ungroup data stored using one file 
     *
     * This is a simple archive format without any compression. 
     * It's used to group all parameters of a task. 
     *
     * The format is really simple : 
     * an header of 4 byte 
     * 0x00000000 ARC1
     * 0x00000004 Entry1
     * 0x???????? ...
     * 0x00000000 EntryN 
     * 
     * Entry: 
     * 4  bytes : The size of the entryName 
     * 4  bytes : The size of the data 
     * NE bytes : The entry Name 
     * ND bytes : The data 
     */

    /**
     * A PackEntry is a summary of a file stored into a pack: name, size and offset
     */
    struct PackEntry 
    {
         String d_name;
         uint32 d_size;
         uint32 d_offset; 
    };
    /**
     * A PackIndex object saves the summary of all files stored into a pack
     */
    typedef std::vector<PackEntry> PackIndex;

    class Pack
    {
    public:
        /**
         * Default Contstructor
         */
        Pack(const String& fileName);
        /**
         * Destructor
         */
        ~Pack(void);

        void close(void);
        /** 
         * Add a file to the archive
         */
        bool add(const String& entryName, const String& fileName);
        /**
         * Add data in memroy to the archive 
         */
        bool add(const String& entryName, byte* data, uint32 len);
        /** 
         * Extract all to dir dirName 
         */ 
        bool extractAll(const String& dirName);
        /**
         * Extract one file to dirName
         */
        bool extract(const String& entryName, const String& localName);
        /**
         * Extract one file and load it into memory
         */
        bool extract(const String& entryName, ByteArray& dest);
        /**
         * Create a PackIndex which contains a summary of all files
         * The summary is composed of three information: 
         *   - The name of the file in the pack 
         *   - The size of the file in the pack 
         *   - The offset of the file from the begining of the file 
         * 
         * @return a pointer to an index. 
         * 
         * @note this pointer should be released using destroyIndex() 
         *
         */
        PackIndex* createIndex();
        /**
         * destroy the PackIndex
         */
        void destroyIndex(PackIndex* index);
    protected:
    private:
        /** Store the filedescriptor associated to the pack */
        int fd;
        /** Store wether the last operation is an addition */
        bool addLast;
    };
}

#endif
/*******************************************************************************/




