/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/**
 *******************************************************************************
 * Utility Library 
 *******************************************************************************
 * @ingroup util_application 
 * @file
 * @brief Class PluginFacotry definitions 
 *  
 * - Supports: All 
 * - Created: 2005-08-06
 * - Creator: Olivier Delannoy
 * - $Revision $
 * - Changelog: 
 *    
 *    * 2005-08-06 : Initial version  
 *******************************************************************************
Copyright Olivier Delannoy 2005

Olivier Delannoy <Olivier.Delannoy@gmail.com>

This software is a computer program whose purpose is to [describe
functionalities and technical features of your software].

This software is governed by the CeCILL  license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

 *******************************************************************************/
#ifndef UTIL_PLUGINFACTORY_H
#define UTIL_PLUGINFACTORY_H 1
#include "util_namespace.hh"
#include "Plugin.hh"
#include "Factory.hh"

namespace Util {
    /**
     * @brief Plugin Factory provides access to modules loaded at
     * runtime.
     * 
     * This class is included in the application core set of
     * features. It's a little unpleaseant to make this class
     * depends on the ltdl lib however it will no generate any
     * portability problem because ltdl is portable. 
     *
     * The Plugin factory load at runtime plugin and make sure a
     * module is loaded only once for all. 
     */
    class PluginFactory : public Factory< Plugin >
    {
    public:
        /**
         * Default Constructor
         */
        PluginFactory(void);
        /**
         * Destructor
         */
        virtual ~PluginFactory(void);
        /**
         * Set the DSO loading path 
         */
        void addPluginPath(const String& path);
        
    protected:
        /** 
         * Create the Plugin object
         */
        virtual Plugin* createObject(const String& name);
            
    private:
    };
}
#endif
/*******************************************************************************/      



