/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/**
 *******************************************************************************
 * Utility Library 
 *******************************************************************************
 * @file
 * @ingroup util_base 
 * @brief Class URL declaration
 *  
 * 
 * - Supports: All  
 * - Created: 2006-03-27
 * - Creator: Olivier Delannoy
 * - Last change in revision : $Revision $
 * - Changelog: 
 *    
 *    * 2006-03-27 : Initial version  
 ******************************************************************************
Copyright Olivier Delannoy 2005

Olivier Delannoy <Olivier.Delannoy@gmail.com>

This software is a computer program whose purpose is to [describe
functionalities and technical features of your software].

This software is governed by the CeCILL  license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

 *******************************************************************************/
#ifndef UTIL_URL_HH
#define UTIL_URL_HH 1
#include "util_namespace.hh"
namespace Util 
{
    /**
     * @ingroup util_base 
     * @brief An URL is a class to parse a string supposed to be an
     * URL and extract the various parts. 
     *
     * This class is used to split the url in the various parts
     * composing it. It's an initial version incomplet at the moment.
     * It split a string respecting the following form :
     * protocol://[server[:port]]/path 
     * RFC: http://www.cse.ohio-state.edu/cgi-bin/rfc/rfc1738.html
     * @see URLFieldName 
     * - Supports: All
     * - Author: Olivier Delannoy
     * - Creator: 2006-03-27
     * - $Revision $
     */
    class UTIL_EXPORT URL
    {
    public:
        /**
         * Constructor 
         * @param url the string containing the url 
         */
        URL(const String& url);
        /** 
         * Return the protocol of the url 
         */

        const String& protocol(void) const;
        /** 
         * Return the rest of the url 
         */
        const String& rest(void) const;
        /** 
         * Return a field of the url 
         * @param field the field we are interested in 
         * 
         * @return the field value if its a valid field or an empty string. 
         *
         * @todo Enable error notification (Exception ?) 
         * @see URLFieldName
         */
        String field(URLFieldName field) const;
    protected:
    
    private:
        /** Store the protocol name */
        String mProtocol;
        /** Store the rest of the url */
        String mRest;
    };
}
#endif // UTIL_URL_HH
/*******************************************************************************/



