/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/**
 *******************************************************************************
 * Utility Library 
 *******************************************************************************
 * @file
 * @ingroup util_xml 
 * @brief Class XMLHandler declaration
 *  
 * 
 * - Supports: All  
 * - Created: 2005-08-09
 * - Creator: Olivier Delannoy
 * - Last change in revision : $Revision $
 * - Changelog: 
 *    
 *    * 2005-08-09 : Initial version  
 ******************************************************************************
Copyright Olivier Delannoy 2005

Olivier Delannoy <Olivier.Delannoy@gmail.com>

This software is a computer program whose purpose is to [describe
functionalities and technical features of your software].

This software is governed by the CeCILL  license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

 *******************************************************************************/
#ifndef UTIL_XMLHANDLER_HH
#define UTIL_XMLHANDLER_HH 1
#include "util_namespace.hh"
#include "Properties.hh"
namespace Util 
{
    /**
     * @ingroup util_xml 
     * @brief XML Handler define the behaviour of parsing an XML document. 
     * 
     * XMLHandler defines the interface used to analyse XML document
     * and generate events from them. This handler class is a
     * simplified version of the SAX API. It triggers only elements
     * and text node. Each XML Document type requires it's own
     * Handler. The default handler implementation does nothingg at all. 
     * 
     * - Supports: All
     * - Author: Olivier Delannoy
     * - Creator: 2005-08-09
     * - $Revision $
     */
    class UTIL_EXPORT XMLHandler
    {
    public:
        /**
         * Constructor 
         */
        XMLHandler(void);
        /**
         * Destructor 
         */
        virtual ~XMLHandler(void);
        
        /**
         * Trigger element's begining event encounters in the document 
         */
        virtual void elementStart(const String& elementName, const Properties &attributes);
        /**
         * Trigger element's begining event encounters in the document 
         */
        virtual void elementEnd(const String& elementName);
        /**
         * Trigger text node event encounters in the document 
         */
        virtual void text(const String& text);
    protected:  
    private:
        /** 
         * Disabled Copy constructor 
         * @param src a reference to the object to copy 
         */
        XMLHandler(XMLHandler& src);

        /**
         * Disabled Assignment operator 
         *
         * @param src a reference to the object to assign from 
         * 
         * @return a reference to the current object 
         */
        XMLHandler& operator=(XMLHandler& src);
    };
}
#endif // UTIL_XMLHANDLER_HH
/*******************************************************************************/



