/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/**
 *******************************************************************************
 * Utility Library 
 *******************************************************************************
 * @file
 * @ingroup util_xml 
 * @brief Class XMLParser declaration
 *  
 * 
 * - Supports: All  
 * - Created: 2005-08-09
 * - Creator: Olivier Delannoy
 * - Last change in revision : $Revision $
 * - Changelog: 
 *    
 *    * 2005-08-09 : Initial version  
 ******************************************************************************
Copyright Olivier Delannoy 2005

Olivier Delannoy <Olivier.Delannoy@gmail.com>

This software is a computer program whose purpose is to [describe
functionalities and technical features of your software].

This software is governed by the CeCILL  license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

 *******************************************************************************/
#ifndef UTIL_XMLPARSER_HH
#define UTIL_XMLPARSER_HH 1
#include "util_namespace.hh"
#include "XMLHandler.hh"
#include "ByteArray.hh" 
#include "Exception.hh"
namespace Util 
{
    class XMLParserImpl;
    //class XMLParserFactory;
    /**
     * @ingroup util_xml 
     * @brief XMLParser smart pointer to implementation 
     *
     * This class is just a kind of smart pointer for XMLParser 
     *
     * - Supports: All 
     * - Author: Olivier Delannoy 
     * - Creator: 2005-08-09
     * - $Revision $
     */
    class UTIL_EXPORT XMLParser
    {
    public:
        /** 
         * Get the name of the parser associated 
         */
        const String& name(void) const;
        /** 
         * Parse and XML Document 
         *
         * @param handler the handler for SAX events 
         * @param data the xml document 
         * @param xsd the xsd document 
         */
        void parse(XMLHandler& handler, const ByteArray& data, const ByteArray* const xsd = 0);
        /** 
         * Parse and XML Document 
         *
         * @param handler the handler for SAX events 
         * @param data name of the file of the xml document 
         * @param xsd the xsd document 
         */
        void parse(XMLHandler& handler, const String& dataFilename, const ByteArray* const xsd = 0);
        /** 
         * Parse and XML Document 
         *
         * @param handler the handler for SAX events 
         * @param data the xml document 
         * @param xsd name of the file of the xsd document 
         */
        void parse(XMLHandler& handler, const ByteArray& data, const String& xsdFilename);
        /** 
         * Parse and XML Document 
         *
         * @param handler the handler for SAX events 
         * @param dataFilename name of the file of the xml document 
         * @param xsdfilename name of the file of the xsd document 
         */
        void parse(XMLHandler& handler, const String& dataFilename, const String& xsdFilename);
        
    protected:
        
    private:
        /** 
         * Constructor 
         * @param impl the implementation of the parser
         */
        XMLParser(XMLParserImpl* impl);
        /** Store a pointer to the parser */
        XMLParserImpl* mParser;
        friend class XMLParserFactory;
    };
}
#endif // UTIL_XMLPARSERIMPL_HH
/*******************************************************************************/



