/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/**
 *******************************************************************************
 * Utility Library 
 *******************************************************************************
 * @file
 * @brief Class XMLParserFactory declaration
 *  
 * 
 * - Supports: All  
 * - Created: 2006-03-24
 * - Creator: Olivier Delannoy
 * - Last change in revision : $Revision $
 * - Changelog: 
 *    
 *    * 2006-03-24 : Initial version  
 ******************************************************************************
Copyright Olivier Delannoy 2005

Olivier Delannoy <Olivier.Delannoy@gmail.com>

This software is a computer program whose purpose is to [describe
functionalities and technical features of your software].

This software is governed by the CeCILL  license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

 *******************************************************************************/
#ifndef UTIL_XMLPARSERFACTORY_HH
#define UTIL_XMLPARSERFACTORY_HH 1
#include "util_namespace.hh"
#include "Singleton.hh" 
#include "XMLParser.hh" 
#include <list>
namespace Util 
{
    class XMLParserImpl;
    /**
     * @brief XML parser factory is used to spawn parser handler and
     * parser implementation on request.
     *
     * The aim of this factory is to do its best to satisfy the user
     * request. In order to do so it manage several XML Parser and
     * instanciate the one needed depending on the feature requested
     * as well as the availability of each parser which may depends
     * from a build to another. 
     *
     * 
     * - Supports: All
     * - Author: Olivier Delannoy
     * - Creator: 2006-03-24
     * - $Revision $
     */
    class UTIL_EXPORT XMLParserFactory : public Singleton<XMLParserFactory> 
    {
    public:
        /**
         * Constructor 
         */
        XMLParserFactory(void);
        /**
         *  Destructor 
         */
        virtual ~XMLParserFactory(void);
        /** 
         * Create a new parser if needed or reuse an existring one 
         * 
         * @param name the name of the parser expected, if not
         * available, the library will try the next parser. 
         * 
         * @return a valid parser handler 
         * 
         */ 
        XMLParser createByName(const String& name);
        /**
         * Create a new parser depending on the requirement of the user 
         * 
         * @param properties BitWise flag on parser option at the
         * moment only validation flag is available
         * - UTIL_XML_PARSER_BEST: Ask to select the best parser depending on the other flags 
         * - UTIL_XML_PARSER_DEFAULT:  Ask for the parser specified at installation 
         * - UTIL_XML_PARSER_VALIDATE: Ask for a validating parser (if available)
         *
         * @return a valid parser handler
         */
        XMLParser create(XMLParserFlag flag = UTIL_XML_PARSER_DEFAULT);
        
        using Singleton<XMLParserFactory>::getSingleton;
        using Singleton<XMLParserFactory>::getSingletonPtr;
    protected:
    
    private:
        /** Typename for the list of loaded parser */
        typedef std::list<XMLParserImpl*> XMLParserImplList;
        /** 
         * Store all parsers loaded 
         */
        XMLParserImplList mParsers;
        /** 
         * Store the default parser has specified during the configure step 
         */
        XMLParserImpl* mDefault;
    };
}
#endif // UTIL_XMLPARSERFACTORY_HH
/*******************************************************************************/



