/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/**
 *******************************************************************************
 * Utility Library 
 *******************************************************************************
 * @file
 * @ingroup util_xml 
 * @brief Abstract Class XMLParser declaration
 *  
 * 
 * - Supports: All  
 * - Created: 2005-08-09
 * - Creator: Olivier Delannoy
 * - Last change in revision : $Revision $
 * - Changelog: 
 *    
 *    * 2005-08-09 : Initial version  
 ******************************************************************************
Copyright Olivier Delannoy 2005

Olivier Delannoy <Olivier.Delannoy@gmail.com>

This software is a computer program whose purpose is to [describe
functionalities and technical features of your software].

This software is governed by the CeCILL  license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

 *******************************************************************************/
#ifndef UTIL_XMLPARSERIMPL_HH
#define UTIL_XMLPARSERIMPL_HH 1
#include "util_namespace.hh"
#include "XMLHandler.hh"
#include "ByteArray.hh" 
#include "Exception.hh" 
namespace Util 
{
    /**
     * @ingroup util_xml 
     * @brief XMLParserImpl Abstract class Definition 
     *
     * This class provide an abstract interface common to all parsers
     * available throught the Utility Library. This simplified SAX
     * interface relies on a handler object in charge of constructing
     * the result. The utility library come with a sample XMLHandler
     * used for creating the DOM Like XML document interface.
     *
     *
     * - Supports: All 
     * - Author: Olivier Delannoy 
     * - Creator: 2005-08-09
     * - $Revision $
     */
    class UTIL_EXPORT XMLParserImpl
    {
    public:
        /**
         * Destructor 
         */
        virtual ~XMLParserImpl(void);
        /** 
         * Get the name of the parser
         */
        const String& name(void) const;
        /** 
         * Return true if the parser is a validating parser 
         */
        bool isValidating(void) const;
        /**
         * Initialize the XMLParser implementation ready for use 
         */
        bool initialize(void);
        /**
         * Cleanup the XMLParser implementation ready for use 
         */
        void cleanup(void);
        /**
         * Parse a file from it's filename 
         */
        virtual void parse(XMLHandler& handler, const ByteArray& data, const ByteArray* const xsd = 0) throw(XMLParseException) = 0;
    protected:
        /**
         * Constructor 
         */
        XMLParserImpl(const String& name, bool isValidating = false);
        /**
         * Implementation of parser initialization 
         */
        virtual bool initializeImpl(void) = 0;
        /**
         * Implementation of cleanup implementation 
         */
        virtual void cleanupImpl(void) = 0;
        
    private:
        /** Name of the parser implementation */
        const String mName;
        /** True if the parser has validation support */
        const bool mValidating;
        /** True if the parser has been initialized */
        bool mInitialized;
        
        /* Disabled operation */
        XMLParserImpl(XMLParserImpl& src);
        /* Disabled operation */
        XMLParserImpl& operator=(XMLParserImpl& src);        
    };
}
#endif // UTIL_XMLPARSERIMPL_HH
/*******************************************************************************/



