/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/**
 *******************************************************************************
 * Utility Library 
 *******************************************************************************
 * @file
 * @brief Class XercesXMLParser declaration
 *  
 * This class is strongly inspired from Paul Turner. 
 * 
 * - Supports: All  
 * - Created: 2005-08-11
 * - Creator: Olivier Delannoy
 * - Last change in revision : $Revision $
 * - Changelog: 
 *    
 *    * 2005-08-11 : Initial version  
 ******************************************************************************
Copyright Olivier Delannoy 2005

Olivier Delannoy <Olivier.Delannoy@gmail.com>

This software is a computer program whose purpose is to [describe
functionalities and technical features of your software].

This software is governed by the CeCILL  license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

 *******************************************************************************/
#ifndef UTIL_XERCESXMLPARSER_HH
#define UTIL_XERCESXMLPARSER_HH 1
#include "util_namespace.hh"
#include "XMLParserImpl.hh"
#error "Xerces Parser not yet available" 
//Xerces C includes 
#include <xercesc/framework/MemBufInputSource.hpp>
#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/util/TransService.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/sax2/Attributes.hpp>
#include <xercesc/sax2/DefaultHandler.hpp>
#include <xercesc/sax2/SAX2XMLReader.hpp>
#include <xercesc/sax2/XMLReaderFactory.hpp>

namespace Util 
{
    /**
     * @brief Xerces based parser implementation.
     *
     * Xerces is an implementation of the XML parser implementation
     * that also provides validation services. 
     * 
     * - Supports: All
     * - Author: Olivier Delannoy
     * - Creator: 2005-08-11
     * - $Revision $
     */
    class UTIL_EXPORT XercesXMLParser : public XMLParserImpl
    {
    public:
        /**
         * Destructor 
         */
        virtual ~XercesXMLParser(void);
        /**
         * Parse a file from it's filename 
         */
        virtual void parse(XMLHandler& handler, const String& fileName, const String& schemaName );
        /**
         * Populate attributs blocks 
         * 
         * @param xerces_attr The list of Xerces attributes.
         * 
         * @param attr The resulting property object 
         */
        static void populateAttributesBlock(const XERCES_CPP_NAMESPACE::Attributes& xerces_attr, Properties& attr);
        /**
         * Transcode us_ascii encoded string to local string 
         */
        static String transcodeXmlCharToString(const XERCES_CPP_NAMESPACE::XMLCh* const xmlch_str, unsigned int inputLength);
        /**
         * Create the reader 
         */
        static XERCES_CPP_NAMESPACE::SAX2XMLReader* XercesXMLParser::createReader(XERCES_CPP_NAMESPACE::DefaultHandler& handler);
        /**
         * Initialize the XML Schema 
         */
        static void XercesXMLParser::initialiseSchema(XERCES_CPP_NAMESPACE::SAX2XMLReader* reader, const String& schemaName, const String& xmlFilename)
    protected:
        /** 
         * Initialize the implementation of the parser 
         */
        virtual bool initializeImpl(void);
        /** 
         * Cleanup the parser implementation 
         */
        virtual void cleanupImpl(void);

    private:
        /**
         * Constructor 
         */
        XercesXMLParser(void);
        friend class XMLParserFactory;
    };
}
#endif // UTIL_XERCESXMLPARSER_HH
/*******************************************************************************/



