/* -*- mode: c++; c-file-style: "engine"; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/**
 *******************************************************************************
 * Utility Library 
 *******************************************************************************
 * @ingroup util_tools 
 * @file
 * @brief Class Zip declaration
 *  
 * 
 * - Supports: All  
 * - Created: 2005-08-25
 * - Creator: Olivier DELANNOY
 * - Last change in revision : $Revision $
 * - Changelog: 
 *    
 *    * 2005-08-25 : Initial version  
 ******************************************************************************
Copyright Olivier Delannoy 2005

Olivier Delannoy <Olivier.Delannoy@gmail.com>

This software is a computer program whose purpose is to [describe
functionalities and technical features of your software].

This software is governed by the CeCILL  license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

 *******************************************************************************/
#ifndef UTIL_ZIP_HH
#define UTIL_ZIP_HH 1
#include "util_namespace.hh"
#include <iostream>
namespace Util 
{
    /**
     * @brief Zip file handling
     *
     * This class is available only if the zlib library is
     * available. It relies on the zlib and it's contribution named
     * minizip. Both are distributed under the term of the zlib
     * license. This class consist only in a partial C++ wrapper for
     * the unz C functions. The features currently available in the
     * class are the following:
     *  - Zip archive creation  
     *    - adding file  
     *  - Zip archive reading 
     *    - Extract all files in a directory 
     * With time we expect to provide full support for zip file handling. 
     *  
     *
     * - Supports: All
     * - Author: Olivier DELANNOY
     * - Creator: 2005-08-25
     * - $Revision $
     */
    class UTIL_EXPORT Zip
    {
    public:
        /**
         * Constructor 
         */
        Zip(const String& fileName, bool openRead = true);
        /**
         * Destructor 
         */
        ~Zip(void);
        // Reading mode interface 
        /**
         * Extract all file to a directory
         * @param dirName place extracted file in this directory 
         */
        bool extractAllEntry(const String& dirName);


        // Creation mode interface
        /** 
         * Add a file to the zip archive in the root folder. 
         * Currently we only support flat zip file without folder in it.
         * 
         * @param fileName the name of the file to add. 
         */
        bool addEntry(const String& fileName);
    protected:
    
    private:
        /** Store the opening mode */
        bool mOpenForReading;
        /** Store zip file information */
        void* mZipHandle; // Opaque structure 
        /**
         * Disabled Assignment operator 
         *
         * @param src a reference to the object to assign from 
         * 
         * @return a reference to the current object 
         */
        Zip& operator=(Zip& src);

        /** 
         * Disabled Copy constructor 
         * @param src a reference to the object to copy 
         */
        Zip(Zip& src);
        /** 
         * Disabled default constructor 
         */
        Zip(void);
    };
}
#endif // UTIL_ZIP_HH
/*******************************************************************************/



